import { PerspectiveCamera, StatsGl } from '@react-three/drei';
import { Canvas, useFrame, useThree } from '@react-three/fiber'
import { CuboidCollider, Physics, RigidBody } from '@react-three/rapier';
import { useEffect, useRef, useState } from 'react'
import { DirectionalLight, Euler, PCFSoftShadowMap, VSMShadowMap, Vector3 } from 'three';
import { Player } from './Player';

function HeRotates() {
  const rotation = useRef(0);
  const objRef = useRef<DirectionalLight>(null);
  useFrame((three, dt) => {
    rotation.current += dt;
    const radius = 3;
    const centerX = 0;
    const centerY = 0;
  
    const x = (radius * Math.cos(rotation.current)) + centerX;
    const z = (radius * Math.sin(rotation.current)) + centerY;
    if (objRef.current) {
      objRef.current.position.x = x;
      objRef.current.position.z = z;
    }
  });
  
  return <directionalLight color="white" position={[0, 10, 0]} ref={objRef} intensity={3.0} castShadow={true} />
}

function App() {
  const testRef = useRef();

  const [cubes, setCubes] = useState([{
    size: [1, 1, 1] as [number, number, number],
    position: [0, 1, 0],
    rotation: [0, 0, 0],
    color: `#ff00ff`,
    madeAt: Date.now()
  }]);

  useEffect(() => {
    let newCubes = [];
    for (let i = 0; i < 100; i++) {
      let size = (Math.random() * 20) + 0.5
      newCubes.push({
        size: [
          size, size, size
        ],
        position: [
          (Math.random() * 400) - 200,
          (Math.random() * 20) + 1,
          (Math.random() * 400) - 200,
        ],
        rotation: [
          (Math.random() * 2 * Math.PI) - Math.PI,
          (Math.random() * 2 * Math.PI) - Math.PI,
          (Math.random() * 2 * Math.PI) - Math.PI
        ],
        color: `#${Math.floor(Math.random() * 256).toString(16).padStart(2, '0')}${Math.floor(Math.random() * 256).toString(16).padStart(2, '0')}${Math.floor(Math.random() * 256).toString(16).padStart(2, '0')}`,
        madeAt: `${i} + ${Date.now()}`
      })
    }

    setCubes(c => ([...c, ...newCubes]));
  }, []);

  window.amogus = testRef;
  return (
    <>
      <Canvas style={{
        height: '100%'
      }}
      shadows={{
        type: PCFSoftShadowMap,
      }}
      camera={{
        fov: 90
      }}
      >
        <StatsGl />
        <ambientLight intensity={0.1} />
        <HeRotates />
        <Physics
        gravity={[0, -30, 0]}
        debug
        >
          <Player />
          <RigidBody friction={0} enabledRotations={[true, true, true]} mass={2}>
            <mesh position={new Vector3(0, 1.5, 0)} rotation={new Euler(-Math.PI/4, 0, -Math.PI/4)}  castShadow={true}>
              <boxGeometry args={[1, 1, 1]} />
              <meshStandardMaterial color={'orange'} />
            </mesh>
          </RigidBody>
          <RigidBody ref={testRef} friction={0} enabledRotations={[true, true, true]} mass={5000} angularVelocity={[10, 40, 10]}>
            <mesh position={new Vector3(3, 1.5, 0)} rotation={new Euler(-Math.PI/4, 0, -Math.PI/4)}  castShadow={true}>
              <boxGeometry args={[1, 1, 1]} />
              <meshStandardMaterial color={'red'} />
            </mesh>
          </RigidBody>
          {cubes.map(v => {
            return <RigidBody key={v.madeAt} density={1} angularVelocity={[10, 10, 10]} friction={0}>
              <mesh position={new Vector3(...v.position)} rotation={new Euler(...v.rotation)} castShadow={true}>
                <boxGeometry args={v.size} />
                <meshStandardMaterial color={v.color} />
              </mesh>
            </RigidBody>
          })}
          <CuboidCollider args={[300, 1, 300]} position={[0, -1, 0]} friction={2}/>
          <mesh rotation={new Euler(-Math.PI/2, 0, 0)} receiveShadow={true}>
            <planeGeometry args={[600, 600]} />
            <meshStandardMaterial color={'white'} />
          </mesh>
        </Physics>
      </Canvas>
    </>
  )
}

export default App
