import { PerspectiveCamera } from "@react-three/drei";
import { useFrame, useThree } from "@react-three/fiber";
import { CapsuleCollider, CylinderCollider, RapierRigidBody, RigidBody, euler, quat } from "@react-three/rapier";
import { useRef } from "react";
import { Euler, PerspectiveCamera as PerspectiveCameraImpl, Quaternion, Vector3 } from "three";

export function Player() {
    const pos = useRef(new Vector3(5, 5, 0));
    const rot = useRef(new Euler(0, 0, 0));
    const rbRef = useRef<RapierRigidBody>(null);
    const camRef = useRef<PerspectiveCameraImpl>(null);

    useFrame((state, dt) => {
        if (!camRef.current || !rbRef.current) return;

        let gps = navigator.getGamepads();
        if (gps.length < 1) return;
        
        const axes = gps[0]!.axes;

        const [motionZ, motionX, viewSideways, viewVertical] = axes.map(val => Math.abs(val) < 0.5 ? 0 : val);

        if (Math.abs(viewVertical) > 0) {
            let goal = camRef.current.rotation.x - (viewVertical * dt * 2);

            camRef.current.rotation.x = Math.max(-Math.PI / 2, Math.min(Math.PI / 2, goal))
        }

        if (Math.abs(viewSideways) > 0) {
            let rot = quat(rbRef.current.rotation());
            rot = rot.multiply(quat().setFromEuler(new Euler(0, (-viewSideways * dt * 2), 0)));
            rbRef.current.setRotation(rot, true);
        }

        let rot = quat(rbRef.current.rotation());

        let desiredMotionVector = new Vector3(
            motionZ * 300, 
            0, 
            motionX * 300
        );
        desiredMotionVector.applyQuaternion(rot);

        rbRef.current.applyImpulseAtPoint(
            desiredMotionVector, 
            rbRef.current.translation(),
            true
        );

        if (gps[0].buttons[0].pressed) {
            rbRef.current.applyImpulseAtPoint(
                new Vector3(0, 500, 0),
                rbRef.current.translation(),
            true
            );
        }

        // let linvel = rbRef.current.linvel();
        // linvel.x = Math.max(-1, Math.min(1, linvel.x));
        // linvel.z = Math.max(-1, Math.min(1, linvel.z));

        // rbRef.current.setLinvel(linvel, true);
    });

    return <>
        <RigidBody ref={rbRef} position={pos.current} rotation={rot.current} enabledRotations={[false, true, false]} angularDamping={50.0} density={1}>
            <PerspectiveCamera ref={camRef} makeDefault fov={90} position={[0, 2, 10]} />
            <CapsuleCollider args={[2, 3]} position={[0, 0, 0]}/>
        </RigidBody>
    </>
}